' bootstrap structural extended VAR impulse responses for Blanchard-Perotti model, to be run after var_p and var_pc
' written by C.A.Favero 4/3/2007

'delete rsp_lvl_upp* rsp_lvl_med* rsp_lvl_low* rsp_acc_upp* rsp_acc_med* rsp_acc_upp*
' set response horizon
!hrz = 20

smpl 1981:1 2007:1
var var_p.ls 1 4 l_gg l_tt l_yy infl_q i @ c

' get residuals to bootstrap
var_p.makeresid(n=gres) res_l_gg res_l_tt res_l_yy  res_infl_q res_i

' make model to generate bootstrap data
model model_bt
model_bt.merge var_p
model_bt.append dy_i=dy_i(-1)*(1+0.25*i)/(1+l_yy-l_yy(-1)+infl_q) +0.25*(exp(l_gg)-exp(l_tt))/(exp(l_yy))

' assign add factors for bootstrap residuals
model_bt.addassign(i)  l_gg l_tt l_yy infl_q i

'set random number generator
rndseed(type=mt) 1234567

'declare storage matrices
!reps = 500
for !i=1 to 12
	matrix(!hrz,!reps) brsp_lvl{!i}
	matrix(!hrz,!reps) brsp_acc{!i}
next

'temporary working vector
vector vtmp
'count number of cases that converged
scalar cnt = 0

'bootstrap loop
for !i=1 to !reps
'bootstrap residuals
smpl @all if res_l_gg<>na
gres.resample l_gg_a l_tt_a l_yy_a infl_q_a i_a

'generate bootstrap data
model_bt.solve

'estimate XVAR  with bootstrap data
smpl 1981:1 2007:1
var var_2.ls 1 4 l_gg_0 l_tt_0 l_yy_0 infl_q_0 i_0 @  c 

'structuralization
var_2.cleartext(svar)
var_2.append(svar) @e1=-0.5*@e4 +c(1)*@u1
var_2.append(svar) @e2=1.97*@e3+1.40*@e4+c(2)*@u1+c(3)*@u2
var_2.append(svar) @e3=c(4)*@e1+c(5)*@e2+c(6)*@u3
var_2.append(svar) @e4=c(7)*@e1+c(8)*@e2+c(9)*@e3+c(10)*@u4
var_2.append(svar) @e5=c(11)*@e1+c(12)*@e2+c(13)*@e3+c(14)*@e4+c(15)*@u5

' estimate structural factorization
var_2.svar(rtype=text,conv=1e-5)

matrix MAT_A=var_2.@svaramat
matrix MAT_B=var_2.@svarbmat
matrix mat_c=@inverse(mat_a)*mat_b
model model_blc
model_blc.merge var_2
model_blc.append dy_i_0=dy_i_0(-1)*(1+0.25*i_0)/(1+l_yy_0-l_yy_0(-1)+infl_q_0) +0.25*(exp(l_gg_0)-exp(l_tt_0))/(exp(l_yy_0))

model_blc.append assign @all  _fbl
model model_alc
model_alc.append assign @all  _fal

model_alc.append l_gg_0 = var_2.c(1,1)*l_gg_0(-1) + var_2.c(1,2)*l_gg_0(-2) + var_2.c(1,3)*l_gg_0(-3) + var_2.c(1,4)*l_gg_0(-4) + var_2.c(1,5)*l_tt_0(-1) + var_2.c(1,6)*l_tt_0(-2) + var_2.c(1,7)*l_tt_0(-3) + var_2.c(1,8)*l_tt_0(-4) + var_2.c(1,9)*l_yy_0(-1) + var_2.c(1,10)*l_yy_0(-2) + var_2.c(1,11)*l_yy_0(-3) + var_2.c(1,12)*l_yy_0(-4) + var_2.c(1,13)*infl_q_0(-1) + var_2.c(1,14)*infl_q_0(-2) + var_2.c(1,15)*infl_q_0(-3) + var_2.c(1,16)*infl_q_0(-4) + var_2.c(1,17)*i_0(-1) + var_2.c(1,18)*i_0(-2) + var_2.c(1,19)*i_0(-3) + var_2.c(1,20)*i_0(-4) + var_2.c(1,21) +mat_c(1,1)*shock1+mat_c(1,2)*shock2

model_alc.append l_tt_0 = var_2.c(2,1)*l_gg_0(-1) + var_2.c(2,2)*l_gg_0(-2) + var_2.c(2,3)*l_gg_0(-3) + var_2.c(2,4)*l_gg_0(-4) + var_2.c(2,5)*l_tt_0(-1) + var_2.c(2,6)*l_tt_0(-2) + var_2.c(2,7)*l_tt_0(-3) + var_2.c(2,8)*l_tt_0(-4) + var_2.c(2,9)*l_yy_0(-1) + var_2.c(2,10)*l_yy_0(-2) + var_2.c(2,11)*l_yy_0(-3) + var_2.c(2,12)*l_yy_0(-4) + var_2.c(2,13)*infl_q_0(-1) + var_2.c(2,14)*infl_q_0(-2) + var_2.c(2,15)*infl_q_0(-3) + var_2.c(2,16)*infl_q_0(-4) + var_2.c(2,17)*i_0(-1) + var_2.c(2,18)*i_0(-2) + var_2.c(2,19)*i_0(-3) + var_2.c(2,20)*i_0(-4) + var_2.c(2,21) + +mat_c(2,1)*shock1+mat_c(2,2)*shock2

model_alc.append l_yy_0 = var_2.c(3,1)*l_gg_0(-1) + var_2.c(3,2)*l_gg_0(-2) + var_2.c(3,3)*l_gg_0(-3) + var_2.c(3,4)*l_gg_0(-4) + var_2.c(3,5)*l_tt_0(-1) + var_2.c(3,6)*l_tt_0(-2) + var_2.c(3,7)*l_tt_0(-3) + var_2.c(3,8)*l_tt_0(-4) + var_2.c(3,9)*l_yy_0(-1) + var_2.c(3,10)*l_yy_0(-2) + var_2.c(3,11)*l_yy_0(-3) + var_2.c(3,12)*l_yy_0(-4) + var_2.c(3,13)*infl_q_0(-1) + var_2.c(3,14)*infl_q_0(-2) + var_2.c(3,15)*infl_q_0(-3) + var_2.c(3,16)*infl_q_0(-4) + var_2.c(3,17)*i_0(-1) + var_2.c(3,18)*i_0(-2) + var_2.c(3,19)*i_0(-3) + var_2.c(3,20)*i_0(-4) + var_2.c(3,21) +mat_c(3,1)*shock1+mat_c(3,2)*shock2

model_alc.append infl_q_0 = var_2.c(4,1)*l_gg_0(-1) + var_2.c(4,2)*l_gg_0(-2) + var_2.c(4,3)*l_gg_0(-3) + var_2.c(4,4)*l_gg_0(-4) + var_2.c(4,5)*l_tt_0(-1) + var_2.c(4,6)*l_tt_0(-2) + var_2.c(4,7)*l_tt_0(-3) + var_2.c(4,8)*l_tt_0(-4) + var_2.c(4,9)*l_yy_0(-1) + var_2.c(4,10)*l_yy_0(-2) + var_2.c(4,11)*l_yy_0(-3) + var_2.c(4,12)*l_yy_0(-4) + var_2.c(4,13)*infl_q_0(-1) + var_2.c(4,14)*infl_q_0(-2) + var_2.c(4,15)*infl_q_0(-3) + var_2.c(4,16)*infl_q_0(-4) + var_2.c(4,17)*i_0(-1) + var_2.c(4,18)*i_0(-2) + var_2.c(4,19)*i_0(-3) + var_2.c(4,20)*i_0(-4) + var_2.c(4,21) +mat_c(4,1)*shock1+mat_c(4,2)*shock2

model_alc.append i_0 = var_2.c(5,1)*l_gg_0(-1) + var_2.c(5,2)*l_gg_0(-2) + var_2.c(5,3)*l_gg_0(-3) + var_2.c(5,4)*l_gg_0(-4) + var_2.c(5,5)*l_tt_0(-1) + var_2.c(5,6)*l_tt_0(-2) + var_2.c(5,7)*l_tt_0(-3) + var_2.c(5,8)*l_tt_0(-4) + var_2.c(5,9)*l_yy_0(-1) + var_2.c(5,10)*l_yy_0(-2) + var_2.c(5,11)*l_yy_0(-3) + var_2.c(5,12)*l_yy_0(-4) + var_2.c(5,13)*infl_q_0(-1) + var_2.c(5,14)*infl_q_0(-2) + var_2.c(5,15)*infl_q_0(-3) + var_2.c(5,16)*infl_q_0(-4) + var_2.c(5,17)*i_0(-1) + var_2.c(5,18)*i_0(-2) + var_2.c(5,19)*i_0(-3) + var_2.c(5,20)*i_0(-4) + var_2.c(5,21) +mat_c(5,1)*shock1+mat_c(5,2)*shock2

model_alc.append dy_i_0=dy_i_0(-1)*(1+0.25*i_0)/(1+l_yy_0-l_yy_0(-1)+infl_q_0) +0.25*(exp(l_gg_0)-exp(l_tt_0))/(exp(l_yy_0))
smpl @all
series shock1=0
series shock2=0
smpl 2007:1+1 2007:1+1
shock1=1
shock2=0

smpl 2007:1+1 2007:1+!hrz
model_blc.solve
model_alc.solve
genr re_pc_lgg_s_lgg=l_gg_0_fal- l_gg_0_fbl
genr re_pc_ltt_s_lgg=l_tt_0_fal- l_tt_0_fbl
genr re_pc_lyy_s_lgg=l_yy_0_fal- l_yy_0_fbl
genr re_pc_infl_q_s_lgg=infl_q_0_fal- infl_q_0_fbl
genr re_pc_i_s_lgg=i_0_fal- i_0_fbl
genr re_pc_dy_i_s_lgg=dy_i_0_fal- dy_i_0_fbl


smpl 2007:1+1 2007:1+1
genr acc_re_pc_lgg_s_lgg=re_pc_lgg_s_lgg
genr acc_re_pc_ltt_s_lgg=re_pc_ltt_s_lgg
genr acc_re_pc_lyy_s_lgg=re_pc_lyy_s_lgg
genr acc_re_pc_infl_q_s_lgg=re_pc_infl_q_s_lgg
genr acc_re_pc_i_s_lgg=re_pc_i_s_lgg
genr acc_re_pc_dy_i_s_lgg=re_pc_dy_i_s_lgg


smpl 2007:1+2 2007:1+!hrz
series acc_re_pc_lgg_s_lgg=acc_re_pc_lgg_s_lgg(-1)+re_pc_lgg_s_lgg
series acc_re_pc_ltt_s_lgg=acc_re_pc_ltt_s_lgg(-1)+re_pc_ltt_s_lgg
series acc_re_pc_lyy_s_lgg=acc_re_pc_lyy_s_lgg(-1)+re_pc_lyy_s_lgg
series acc_re_pc_infl_q_s_lgg=acc_re_pc_infl_q_s_lgg(-1)+re_pc_infl_q_s_lgg
series acc_re_pc_i_s_lgg=acc_re_pc_i_s_lgg(-1)+re_pc_i_s_lgg
series acc_re_pc_dy_i_s_lgg=acc_re_pc_dy_i_s_lgg(-1)+re_pc_dy_i_s_lgg

smpl 2007:1+1 2007:1+1
shock1=0
shock2=1

smpl 2007:1+1 2007:1+!hrz
model_blc.solve
model_alc.solve
genr re_pc_lgg_s_ltt=l_gg_0_fal- l_gg_0_fbl
genr re_pc_ltt_s_ltt=l_tt_0_fal- l_tt_0_fbl
genr re_pc_lyy_s_ltt=l_yy_0_fal- l_yy_0_fbl
genr re_pc_infl_q_s_ltt=infl_q_0_fal- infl_q_0_fbl
genr re_pc_i_s_ltt=i_0_fal- i_0_fbl
genr re_pc_dy_i_s_ltt=dy_i_0_fal- dy_i_0_fbl


smpl 2007:1+1 2007:1+1
genr acc_re_pc_lgg_s_ltt=re_pc_lgg_s_ltt
genr acc_re_pc_ltt_s_ltt=re_pc_ltt_s_ltt
genr acc_re_pc_lyy_s_ltt=re_pc_lyy_s_ltt
genr acc_re_pc_infl_q_s_ltt=re_pc_infl_q_s_ltt
genr acc_re_pc_i_s_ltt=re_pc_i_s_ltt
genr acc_re_pc_dy_i_s_ltt=re_pc_dy_i_s_ltt



smpl 2007:1+2 2007:1+!hrz
series acc_re_pc_lgg_s_ltt=acc_re_pc_lgg_s_ltt(-1)+re_pc_lgg_s_ltt
series acc_re_pc_ltt_s_ltt=acc_re_pc_ltt_s_ltt(-1)+re_pc_ltt_s_ltt
series acc_re_pc_lyy_s_ltt=acc_re_pc_lyy_s_ltt(-1)+re_pc_lyy_s_ltt
series acc_re_pc_infl_q_s_ltt=acc_re_pc_infl_q_s_ltt(-1)+re_pc_infl_q_s_ltt
series acc_re_pc_i_s_ltt=acc_re_pc_i_s_ltt(-1)+re_pc_i_s_ltt
series acc_re_pc_dy_i_s_ltt=acc_re_pc_dy_i_s_ltt(-1)+re_pc_dy_i_s_ltt

smpl 2007:1+1 2007:1+!hrz
'get level and accumulated bootstrap responses
group imp_varpc
imp_varpc.add  re_pc_lgg_s_lgg re_pc_ltt_s_lgg re_pc_lyy_s_lgg re_pc_infl_q_s_lgg re_pc_i_s_lgg re_pc_dy_i_s_lgg re_pc_lgg_s_ltt re_pc_ltt_s_ltt re_pc_lyy_s_ltt re_pc_infl_q_s_ltt re_pc_i_s_ltt re_pc_dy_i_s_ltt
stom(imp_varpc,tmp_lvl)
delete imp_varpc
group acc_imp_varpc
acc_imp_varpc.add  acc_re_pc_lgg_s_lgg acc_re_pc_ltt_s_lgg acc_re_pc_lyy_s_lgg acc_re_pc_infl_q_s_lgg acc_re_pc_i_s_lgg acc_re_pc_dy_i_s_lgg acc_re_pc_lgg_s_ltt acc_re_pc_ltt_s_ltt acc_re_pc_lyy_s_ltt acc_re_pc_infl_q_s_ltt acc_re_pc_i_s_ltt acc_re_pc_dy_i_s_ltt
stom(acc_imp_varpc,tmp_acc)
delete acc_imp_varpc
'and store in appropriate position
		for !j=1 to @columns(rsp_lvl_bpsim)
			vtmp = @columnextract(tmp_lvl,!j)
			colplace(brsp_lvl{!j}, vtmp, !i)
			vtmp = @columnextract(tmp_acc,!j)
			colplace(brsp_acc{!j}, vtmp, !i)
next
delete tmp_lvl tmp_acc
cnt=cnt+1
next
'find bootstrap percentiles for each horizon
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_lvl_bp_upp_2
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_lvl_bp_med_2
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_lvl_bp_low_2
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_acc_bp_upp_2
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_acc_bp_med_2
matrix(!hrz,@columns(rsp_lvl_bpsim)) rsp_acc_bp_low_2
rowvector rtmp
for !i=1 to !hrz
	for !j=1 to @columns(rsp_lvl_bpsim)
		rtmp = @rowextract(brsp_lvl{!j},!i)
		rsp_lvl_bp_upp_2(!i,!j) = @quantile(rtmp,0.833)
		rsp_lvl_bp_med_2(!i,!j) = @quantile(rtmp,0.5)
		rsp_lvl_bp_low_2(!i,!j) = @quantile(rtmp,0.167)
		rtmp = @rowextract(brsp_acc{!j},!i)
		rsp_acc_bp_upp_2(!i,!j) = @quantile(rtmp,0.833)
		rsp_acc_bp_med_2(!i,!j) = @quantile(rtmp,0.833)
		rsp_acc_bp_low_2(!i,!j) = @quantile(rtmp,0.167)
	next
next

smpl 1981:1 2007:1
scalar factor2=0.01*(@mean(yy)/@mean(tt))/rsp_lvl_bp_med_1(1,8)
smpl 1980:4+1 1980:4+!hrz
matrix rsp_lvl_bp_med_2_tr= rsp_lvl_bp_med_2*factor2
matrix rsp_lvl_bp_low_2_tr=rsp_lvl_bp_low_2*factor2
matrix rsp_lvl_bp_upp_2_tr= rsp_lvl_bp_upp_2*factor2



delete res_l_gg res_l_tt res_l_yy  res_infl_q res_i
delete re_*  acc_* brsp_*
delete dy_i_0* i_0* infl_q_0* l_gg_0* l_tt_0* l_yy_0* gres
delete i_a infl_q_a l_gg_a l_tt_a l_yy_a vtmp rtmp shock1 shock2

